from unittest import TestCase


class GithubApiTests(TestCase):
    def test_clone_github_repository(self):
        pass

    def test_create_feature_branch(self):
        pass

    def test_run_script_inside_cloned_repository(self):
        pass

    def test_commit_changes_made_to_cloned_repository(self):
        pass

    def test_push_commits(self):
        pass

    def test_create_pull_request(self):
        pass

    def test_execution_logs(self):
        # Test that pull request link is present in the logs
        # Test that error Tracebacks are in the logs
        pass
