from unittest import TestCase

from gitmultirepoupdater.cli import main


# class CommandLineArgumentParsingTests(TestCase):
#     def test_repositories_parsing(self):
#         args = main(["-r", "foo"])
#         self.assertEqual(args.repos, ["foo"])
